#include "Dict.h"

#include <catch.h>
#include <iostream>
#include <string>

using namespace std;
using namespace dictionary;

TEST_CASE("Member_empty")
{
    Dict<string, int> dict;
    CHECK_FALSE(dict.has_key("a"));
}

TEST_CASE("Find_not_found_is_null")
{
    Dict<string, int> dict;
    CHECK(dict.find("a") == nullptr);
}

TEST_CASE("Insert_first_element_then_lookup")
{
    Dict<string, int> dict;
    dict.insert("b", 5);

    CHECK_FALSE(dict.has_key("a"));
    CHECK(dict.has_key("b"));
    CHECK(5 == dict["b"]);
}

TEST_CASE("Insert_several")
{
    Dict<string, int> dict;
    dict.insert("d", 4);
    dict.insert("b", 2);
    dict.insert("a", 1);

    CHECK(dict.has_key("a"));
    CHECK(dict.has_key("b"));
    CHECK_FALSE(dict.has_key("c"));
    CHECK(dict.has_key("d"));
    CHECK_FALSE(dict.has_key("e"));
    CHECK_FALSE(dict.has_key("f"));
    CHECK_FALSE(dict.has_key("g"));

    dict.insert("c", 3);
    dict.insert("f", 6);
    dict.insert("e", 5);
    dict.insert("g", 7);

    CHECK(dict.has_key("a"));
    CHECK(dict.has_key("b"));
    CHECK(dict.has_key("c"));
    CHECK(dict.has_key("d"));
    CHECK(dict.has_key("e"));
    CHECK(dict.has_key("f"));
    CHECK(dict.has_key("g"));
    CHECK_FALSE(dict.has_key(""));
    CHECK_FALSE(dict.has_key("ax"));
    CHECK_FALSE(dict.has_key("bx"));
    CHECK_FALSE(dict.has_key("cx"));
    CHECK_FALSE(dict.has_key("dx"));
    CHECK_FALSE(dict.has_key("ex"));
    CHECK_FALSE(dict.has_key("fx"));
    CHECK_FALSE(dict.has_key("gx"));

    CHECK(1 == dict["a"]);
    CHECK(2 == dict["b"]);
    CHECK(3 == dict["c"]);
    CHECK(4 == dict["d"]);
    CHECK(5 == dict["e"]);
    CHECK(6 == dict["f"]);
    CHECK(7 == dict["g"]);

    CHECK(dict.find("h") == nullptr);

    int* e = dict.find("e");
    CHECK(5 == *e);

    *e = 105;
    CHECK(105 == dict["e"]);

    CHECK(0 == dict["h"]);
}

TEST_CASE("initializer_list<value_type>")
{
    Dict<string, int> dict{{"hello", 5}, {"goodbye", 7}};
}

TEST_CASE("Build from vector iterator")
{
    vector<pair<string, int>> v{{"hello", 5}, {"goodbye", 7}};
    Dict<string, int> dict(v.begin(), v.end());
}

TEST_CASE("iterate over dictionary")
{
    Dict<string, int> dict{{"hello", 5}, {"goodbye", 7}};

    for (const auto& i : dict) { }
}

TEST_CASE("convert iterator to const_iterator")
{
    Dict<string, int> dict;
    typename Dict<string, int>::const_iterator i = dict.begin();
}
