#include "Int_vector.h"

#include <catch.h>

#include <iostream>

TEST_CASE("New_empty")
{
    Int_vector v;
}

TEST_CASE("New_with_capacity")
{
    Int_vector v(16);
}

TEST_CASE("Initializer_list")
{
    Int_vector v{1, 2, 3};

    CHECK(3 == v.size());
    CHECK(1 == v[0]);
    CHECK(2 == v[1]);
    CHECK(3 == v[2]);
}

TEST_CASE("Copy_constructor")
{
    Int_vector u{1, 2, 3};
    Int_vector v(u);
    Int_vector w(v);

    CHECK(u == w);

    v[0] = 5;
    CHECK(u == w);
    CHECK(u != v);
}

TEST_CASE("Assignment_operator")
{
    Int_vector v{1, 2, 3};
    Int_vector w;

    CHECK(v != w);
    w = v;
    CHECK(v == w);
    w.push_back(4);
    CHECK(v != w);
    w = v;
    CHECK(v == w);
}

TEST_CASE("Empty")
{
    Int_vector v;
    CHECK(v.empty());
    v.push_back(0);
    CHECK_FALSE(v.empty());
}

TEST_CASE("Size")
{
    Int_vector v;
    CHECK(0 == v.size());
    v.push_back(1);
    CHECK(1 == v.size());
    v.push_back(2);
    CHECK(2 == v.size());
    v.push_back(3);
    CHECK(3 == v.size());
    v.push_back(4);
    CHECK(4 == v.size());
}

TEST_CASE("Push_back")
{
    Int_vector v;

    v.push_back(5);
    v.push_back(6);
    v.push_back(7);
    v.push_back(8);
    v.push_back(9);
    v.push_back(10);
    v.push_back(11);
    v.push_back(12);
    v.push_back(13);

    CHECK(5 == v[0]);
    CHECK(6 == v[1]);
    CHECK(7 == v[2]);
    CHECK(13 == v[8]);
}

TEST_CASE("Pop_back")
{
    Int_vector v{1, 2, 3};
    CHECK(3 == v.size());
    v.pop_back();
    CHECK(2 == v.size());
    v.pop_back();
    CHECK(1 == v.size());
    v.pop_back();
    CHECK(0 == v.size());
}

TEST_CASE("Clear")
{
    Int_vector v{1, 2, 3};
    CHECK(3 == v.size());
    v.clear();
    CHECK(0 == v.size());
    v.push_back(4);
    CHECK(1 == v.size());
}

TEST_CASE("Front_const")
{
    const Int_vector v{1, 2, 3};
    CHECK(1 == v.front());
}

TEST_CASE("Front_assign")
{
    Int_vector v{1, 2, 3};
    CHECK(1 == v.front());
    v.front() = 10;
    CHECK(10 == v.front());
}

TEST_CASE("Back_const")
{
    const Int_vector v{1, 2, 3};
    CHECK(3 == v.back());
}

TEST_CASE("Back_assign")
{
    Int_vector v{1, 2, 3};
    CHECK(3 == v.back());
    v.back() = 10;
    CHECK(10 == v.back());
}

TEST_CASE("Index_const")
{
    const Int_vector v{1, 2, 3};
    CHECK(1 == v[0]);
    CHECK(2 == v[1]);
    CHECK(3 == v[2]);
}

TEST_CASE("Index_assign")
{
    Int_vector v{1, 2, 3};
    CHECK(1 == v[0]);
    CHECK(2 == v[1]);
    CHECK(3 == v[2]);

    v[1] = 10;

    CHECK(1 == v[0]);
    CHECK(10 == v[1]);
    CHECK(3 == v[2]);
}

TEST_CASE("At_const")
{
    const Int_vector v{1, 2, 3};
    CHECK(1 == v.at(0));
    CHECK(2 == v.at(1));
    CHECK(3 == v.at(2));
    CHECK_THROWS_AS(v.at(3), std::range_error);
}

TEST_CASE("At_assign")
{
    Int_vector v{1, 2, 3};
    CHECK(1 == v.at(0));
    CHECK(2 == v.at(1));
    CHECK(3 == v.at(2));

    v.at(1) = 10;

    CHECK(1 == v.at(0));
    CHECK(10 == v.at(1));
    CHECK(3 == v.at(2));
}

TEST_CASE("Iteration")
{
    Int_vector v;
    v.push_back(5);
    v.push_back(6);
    v.push_back(7);
    v.push_back(8);

    auto i = std::begin(v);
    auto j = std::end(v);
    CHECK(i != j);
    CHECK(5 == *i);

    ++i;
    CHECK(i != j);
    CHECK(6 == *i);

    ++i;
    CHECK(i != j);
    CHECK(7 == *i);

    ++i;
    CHECK(i != j);
    CHECK(8 == *i);

    ++i;
    CHECK(j == i);
}

TEST_CASE("Comparisons")
{
    Int_vector u{1, 2, 3};
    Int_vector v{1, 2, 3, 4};
    Int_vector w{1, 4, 3};

    CHECK_FALSE(u < u);
    CHECK(u < v);
    CHECK(u < w);
    CHECK_FALSE(v < u);
    CHECK_FALSE(v < v);
    CHECK(v < w);
    CHECK_FALSE(w < u);
    CHECK_FALSE(w < v);
    CHECK_FALSE(w < w);

    CHECK(u <= u);
    CHECK(u <= v);
    CHECK(u <= w);
    CHECK_FALSE(v <= u);
    CHECK(v <= v);
    CHECK(v <= w);
    CHECK_FALSE(w <= u);
    CHECK_FALSE(w <= v);
    CHECK(w <= w);

    CHECK_FALSE(u > u);
    CHECK_FALSE(u > v);
    CHECK_FALSE(u > w);
    CHECK(v > u);
    CHECK_FALSE(v > v);
    CHECK_FALSE(v > w);
    CHECK(w > u);
    CHECK(w > v);
    CHECK_FALSE(w > w);

    CHECK(u >= u);
    CHECK_FALSE(u >= v);
    CHECK_FALSE(u >= w);
    CHECK(v >= u);
    CHECK(v >= v);
    CHECK_FALSE(v >= w);
    CHECK(w >= u);
    CHECK(w >= v);
    CHECK(w >= w);
}

TEST_CASE("Reverse_iterator") {
    Int_vector v{1, 2, 3};

    auto i = v.rbegin();
    auto j = v.rend();

    CHECK(i < j);
    CHECK(3 == *i);
    CHECK(1 == i[2]);
    ++i;
    CHECK(2 == *i);
    CHECK(3 == i[-1]);
    ++i;
    CHECK(1 == *i);
    ++i;
    CHECK(i == j);
}
